'use strict';

var view = require('@codemirror/view');
var language = require('@codemirror/language');
var highlight = require('@lezer/highlight');

// Monokai theme colors aligned with the schema
const base00 = '#272822', // Background
base01 = '#f8f8f2', // Foreground
base02 = '#4a4a76', // Selection and Selection Match
base03 = '#88846f', // Comment
base04 = '#f8f8f0', // Cursor
base05 = '#F92672', // Keyword, Storage, Tag
base06 = '#FD971F', // Variable, Parameter
base07 = '#66D9EF', // Function, Type
base08 = '#E6DB74', // String, RegExp
base09 = '#AE81FF', // Constant, Number
base0A = '#A6E22E', // Class, Heading
invalid = '#F44747'; // Invalid
const darkBackground = '#414339', // Dropdown background
highlightBackground = '#3e3d3257', // Line highlight with some opacity
tooltipBackground = base01, // Tooltips using foreground color
cursor = base04, // Caret color
selection = base02; // Selection color
// Define the editor theme styles for Monokai.
const monokaiTheme = view.EditorView.theme({
    '&': {
        color: base01,
        backgroundColor: base00,
    },
    '.cm-content': {
        caretColor: cursor,
    },
    '.cm-cursor, .cm-dropCursor': {
        borderLeftColor: cursor,
    },
    '&.cm-focused > .cm-scroller > .cm-selectionLayer .cm-selectionBackground, .cm-selectionBackground, .cm-content ::selection': {
        backgroundColor: selection,
    },
    '.cm-searchMatch': {
        backgroundColor: base02, // Use the selection color for search matches
        color: base01, // Foreground color for text
        outline: `1px solid ${base03}`, // Outline using comment color
    },
    '.cm-searchMatch.cm-searchMatch-selected': {
        backgroundColor: base05, // Use the keyword color for the selected search match
        color: base00, // Background color for high visibility
    },
    '.cm-panels': {
        backgroundColor: darkBackground,
        color: base01,
    },
    '.cm-panels.cm-panels-top': {
        borderBottom: '2px solid black',
    },
    '.cm-panels.cm-panels-bottom': {
        borderTop: '2px solid black',
    },
    '.cm-activeLine': {
        backgroundColor: highlightBackground,
    },
    '.cm-gutters': {
        backgroundColor: darkBackground, // Using dropdown background for gutter
        color: base03,
    },
    '.cm-tooltip': {
        backgroundColor: tooltipBackground,
    },
}, { dark: true });
// Define the highlighting style for code in the Monokai theme.
const monokaiHighlightStyle = language.HighlightStyle.define([
    { tag: highlight.tags.keyword, color: base05 },
    { tag: [highlight.tags.name, highlight.tags.deleted, highlight.tags.character, highlight.tags.macroName], color: base06 },
    { tag: [highlight.tags.propertyName], color: base07 },
    { tag: [highlight.tags.processingInstruction, highlight.tags.string, highlight.tags.inserted, highlight.tags.special(highlight.tags.string)], color: base08 },
    { tag: [highlight.tags.function(highlight.tags.variableName), highlight.tags.labelName], color: base07 },
    { tag: [highlight.tags.color, highlight.tags.constant(highlight.tags.name), highlight.tags.standard(highlight.tags.name)], color: base09 },
    { tag: [highlight.tags.definition(highlight.tags.name), highlight.tags.separator], color: base06 },
    { tag: [highlight.tags.className], color: base0A },
    { tag: [highlight.tags.number, highlight.tags.changed, highlight.tags.annotation, highlight.tags.modifier, highlight.tags.self, highlight.tags.namespace], color: base09 },
    { tag: [highlight.tags.typeName], color: base07 },
    { tag: [highlight.tags.operator, highlight.tags.operatorKeyword], color: base05 },
    { tag: [highlight.tags.url, highlight.tags.escape, highlight.tags.regexp, highlight.tags.link], color: base08 },
    { tag: [highlight.tags.meta, highlight.tags.comment], color: base03 },
    { tag: highlight.tags.tagName, color: base05 },
    { tag: highlight.tags.strong, fontWeight: 'bold' },
    { tag: highlight.tags.emphasis, fontStyle: 'italic' },
    { tag: highlight.tags.link, textDecoration: 'underline' },
    { tag: highlight.tags.heading, fontWeight: 'bold', color: base0A },
    { tag: [highlight.tags.atom, highlight.tags.bool, highlight.tags.special(highlight.tags.variableName)], color: base06 },
    { tag: highlight.tags.invalid, color: invalid },
    { tag: highlight.tags.strikethrough, textDecoration: 'line-through' },
]);
// Extension to enable the Monokai theme (both the editor theme and the highlight style).
const monokai = [
    monokaiTheme,
    language.syntaxHighlighting(monokaiHighlightStyle),
];

exports.monokai = monokai;
exports.monokaiHighlightStyle = monokaiHighlightStyle;
exports.monokaiTheme = monokaiTheme;
