import { EditorView } from '@codemirror/view';
import { HighlightStyle, syntaxHighlighting } from '@codemirror/language';
import { tags } from '@lezer/highlight';

// Monokai theme colors aligned with the schema
const base00 = '#272822', // Background
base01 = '#f8f8f2', // Foreground
base02 = '#4a4a76', // Selection and Selection Match
base03 = '#88846f', // Comment
base04 = '#f8f8f0', // Cursor
base05 = '#F92672', // Keyword, Storage, Tag
base06 = '#FD971F', // Variable, Parameter
base07 = '#66D9EF', // Function, Type
base08 = '#E6DB74', // String, RegExp
base09 = '#AE81FF', // Constant, Number
base0A = '#A6E22E', // Class, Heading
invalid = '#F44747'; // Invalid
const darkBackground = '#414339', // Dropdown background
highlightBackground = '#3e3d3257', // Line highlight with some opacity
tooltipBackground = base01, // Tooltips using foreground color
cursor = base04, // Caret color
selection = base02; // Selection color
// Define the editor theme styles for Monokai.
const monokaiTheme = /*@__PURE__*/EditorView.theme({
    '&': {
        color: base01,
        backgroundColor: base00,
    },
    '.cm-content': {
        caretColor: cursor,
    },
    '.cm-cursor, .cm-dropCursor': {
        borderLeftColor: cursor,
    },
    '&.cm-focused > .cm-scroller > .cm-selectionLayer .cm-selectionBackground, .cm-selectionBackground, .cm-content ::selection': {
        backgroundColor: selection,
    },
    '.cm-searchMatch': {
        backgroundColor: base02, // Use the selection color for search matches
        color: base01, // Foreground color for text
        outline: `1px solid ${base03}`, // Outline using comment color
    },
    '.cm-searchMatch.cm-searchMatch-selected': {
        backgroundColor: base05, // Use the keyword color for the selected search match
        color: base00, // Background color for high visibility
    },
    '.cm-panels': {
        backgroundColor: darkBackground,
        color: base01,
    },
    '.cm-panels.cm-panels-top': {
        borderBottom: '2px solid black',
    },
    '.cm-panels.cm-panels-bottom': {
        borderTop: '2px solid black',
    },
    '.cm-activeLine': {
        backgroundColor: highlightBackground,
    },
    '.cm-gutters': {
        backgroundColor: darkBackground, // Using dropdown background for gutter
        color: base03,
    },
    '.cm-tooltip': {
        backgroundColor: tooltipBackground,
    },
}, { dark: true });
// Define the highlighting style for code in the Monokai theme.
const monokaiHighlightStyle = /*@__PURE__*/HighlightStyle.define([
    { tag: tags.keyword, color: base05 },
    { tag: [tags.name, tags.deleted, tags.character, tags.macroName], color: base06 },
    { tag: [tags.propertyName], color: base07 },
    { tag: [tags.processingInstruction, tags.string, tags.inserted, /*@__PURE__*/tags.special(tags.string)], color: base08 },
    { tag: [/*@__PURE__*/tags.function(tags.variableName), tags.labelName], color: base07 },
    { tag: [tags.color, /*@__PURE__*/tags.constant(tags.name), /*@__PURE__*/tags.standard(tags.name)], color: base09 },
    { tag: [/*@__PURE__*/tags.definition(tags.name), tags.separator], color: base06 },
    { tag: [tags.className], color: base0A },
    { tag: [tags.number, tags.changed, tags.annotation, tags.modifier, tags.self, tags.namespace], color: base09 },
    { tag: [tags.typeName], color: base07 },
    { tag: [tags.operator, tags.operatorKeyword], color: base05 },
    { tag: [tags.url, tags.escape, tags.regexp, tags.link], color: base08 },
    { tag: [tags.meta, tags.comment], color: base03 },
    { tag: tags.tagName, color: base05 },
    { tag: tags.strong, fontWeight: 'bold' },
    { tag: tags.emphasis, fontStyle: 'italic' },
    { tag: tags.link, textDecoration: 'underline' },
    { tag: tags.heading, fontWeight: 'bold', color: base0A },
    { tag: [tags.atom, tags.bool, /*@__PURE__*/tags.special(tags.variableName)], color: base06 },
    { tag: tags.invalid, color: invalid },
    { tag: tags.strikethrough, textDecoration: 'line-through' },
]);
// Extension to enable the Monokai theme (both the editor theme and the highlight style).
const monokai = [
    monokaiTheme,
    /*@__PURE__*/syntaxHighlighting(monokaiHighlightStyle),
];

export { monokai, monokaiHighlightStyle, monokaiTheme };
